@@ -33,11 +33,13 @@
 #' List Known Character Encodings
 #'
 #' @description
-#' Gives encodings that are supported by \pkg{ICU}.
+#' Gives the list of encodings that are supported by \pkg{ICU}.
 #'
 #' @details
-#' Please note that apart from given encodings, \pkg{ICU} tries to normalize
-#' encoding specifiers. E.g. \code{"UTF8"} is also valid,
+#' Please note that apart from given encoding identifiers and their aliases,
+#' some other specifiers will be available in your conversion tasks.
+#' This is due to the fact that \pkg{ICU} tries to normalize
+#' converter names. E.g. \code{"UTF8"} is also valid,
 #' see \link{stringi-encoding} for more information.
 #'
 #' @param simplified single logical value; return a character vector or a
@@ -50,7 +52,8 @@
 #'  its aliases.
 #'
 #' If \code{simplified} is \code{TRUE}, then the resulting list
-#' is coerced to a character vector, sorted, and its duplicates are removed.
+#' is coerced to a character vector and sorted, and returned with
+#' removed duplicated entries.
 #'
 #' @family encoding_management
 #' @export
@@ -59,25 +62,22 @@ stri_enc_list <- function(simplified=FALSE) {
 
    ret <- .Call("stri_enc_list", PACKAGE="stringi")
    if (simplified)
-      return(stri_sort(unique(unlist(ret))))
+      return(stri_sort(unique(unlist(ret)))) # @TODO: use stri_unique
    else
       return(ret)
 }
 
 
 #' @title
-#' Query Given Character Encoding
+#' Query a Character Encoding
 #'
 #' @description
-#' Gets basic information on a given character encoding.
+#' Gets basic information on a character encoding.
 #'
 #' @details
-#' If the encoding provided is unknown to \pkg{ICU} (see \code{\link{stri_enc_list}}),
-#' an error is generated.
+#' An error is raised if the encoding provided is unknown to \pkg{ICU}
+#' (see \code{\link{stri_enc_list}} for more details)
 #'
-#' If you set a default encoding that is not a superset of ASCII
-#' or it is not an 8-bit encoding, a warning will be generated,
-#' see \link{stringi-encoding} for discussion.
 #'
 #' @param enc \code{NULL} or \code{""} for default encoding,
 #' or a single string with encoding name
@@ -85,23 +85,27 @@ stri_enc_list <- function(simplified=FALSE) {
 #' @return
 #' Returns a list with the following components:
 #' \itemize{
-#' \item \code{Name.friendly} -- Friendly encoding name: MIME Name or JAVA Name
-#'     or \pkg{ICU} Canonical Name
-#'    (selecting the first of supported ones, see below);
+#' \item \code{Name.friendly} -- Friendly encoding name:
+#'     MIME Name or JAVA Name or \pkg{ICU} Canonical Name
+#'    (the first of provided ones is selected, see below);
 #' \item \code{Name.ICU} -- Encoding name as identified by \pkg{ICU};
 #' \item \code{Name.*} -- other standardized encoding names,
 #' e.g. \code{Name.UTR22}, \code{Name.IBM}, \code{Name.WINDOWS},
-#' \code{Name.JAVA}, \code{Name.IANA}, \code{Name.MIME} (some may be not
-#' available for selected  encodings);
+#' \code{Name.JAVA}, \code{Name.IANA}, \code{Name.MIME} (some of them
+#' may be unavailable for all the encodings);
 #' \item \code{ASCII.subset} -- is ASCII a subset of the given encoding?;
 #' \item \code{Unicode.1to1} -- for 8-bit encodings only: are all characters
-#' translated to exactly one Unicode code point and is this translation
-#' well reversible?;
+#' translated to exactly one Unicode code point and is the translation
+#' scheme reversible?;
 #' \item \code{CharSize.8bit} -- is this an 8-bit encoding, i.e. do we have
 #'    \code{CharSize.min == CharSize.max} and \code{CharSize.min == 1}?;
-#' \item \code{CharSize.min} -- minimal number of bytes used to represent a code point;
-#' \item \code{CharSize.max} -- maximal number of bytes used to represent a code point.
+#' \item \code{CharSize.min} -- minimal number of bytes used
+#'    to represent an UChar (in UTF-16, this is not the same as UChar32)
+#' \item \code{CharSize.max} -- maximal number of bytes used
+#'    to represent an UChar (in UTF-16, this is not the same as UChar32,
+#'    i.e. does not reflect the maximal codepoint representation size)
 #' }
+#'
 #' @family encoding_management
 #' @export
 stri_enc_info <- function(enc=NULL) {
@@ -113,8 +117,9 @@ stri_enc_info <- function(enc=NULL) {
 #' Set or Get Default Character Encoding in \pkg{stringi}
 #'
 #' @description
-#' \code{stri_enc_set} sets the encoding used to decode strings
-#' internally (i.e. by \R) marked as Native, see \link{stringi-encoding}.
+#' \code{stri_enc_set} sets the encoding used to re-encode strings
+#' internally (i.e. by \R) declared to be in native encoding,
+#' see \link{stringi-encoding} and \code{\link{stri_enc_mark}}.
 #' \code{stri_enc_get} returns currently used default encoding.
 #'
 #' @details
@@ -122,6 +127,16 @@ stri_enc_info <- function(enc=NULL) {
 #' \code{\link{stri_enc_info}(NULL)$Name.friendly}.
 #'
 #' Note that changing the default encoding may have undesired consequences.
+#' Unless you are an expert user and you know what you are doing,
+#' \code{stri_enc_set} should only be used if \pkg{ICU} fails to detect
+#' your system's encoding correctly (while testing \pkg{stringi}
+#' we only encountered such a situation on a very old Solaris machine, though).
+#' Note that \pkg{ICU} tries to match the encoding part of the \code{LC_CTYPE}
+#' category as given by \code{\link{Sys.getlocale}}.
+#'
+#' If you set a default encoding that is not a superset of ASCII
+#' or it is not an 8-bit encoding, a warning will be generated,
+#' see \link{stringi-encoding} for discussion.
 #'
 #' @param enc single string; character encoding name,
 #' see \code{\link{stri_enc_list}} for the list of supported encodings.
@@ -152,3 +167,52 @@ stri_enc_set <- function(enc) {
 stri_enc_get <- function() {
    stri_enc_info(NULL)$Name.friendly
 }
+
+
+#' @title
+#' Get Declared Encodings of Each String
+#'
+#' @description
+#' Gets declared encodings for each string in a character vector
+#' as seen by \pkg{stringi}.
+#'
+#' @details
+#' According to \code{\link{Encoding}},
+#' \R has a simple encoding marking mechanism:
+#' strings can be declared to be in \code{latin1},
+#' \code{UTF-8} or \code{bytes}.
+#'
+#' Moreover, via the C API we may check whether
+#' a string is in ASCII (\R assumes that this holds iff
+#' all bytes in a string are not greater than 127,
+#' so there is an implicit assumption that your platform uses
+#' an encoding which is an ASCII superset)
+#' or in the system's default (a.k.a. \code{unknown} in \code{\link{Encoding}})
+#' encoding.
+#'
+#' Intuitively, the default encoding should be equivalent to
+#' the one you use when inputting data via keyboard.
+#' In \code{stringi} we assume that such an encoding
+#' is equivalent to the one returned by \code{\link{stri_enc_get}}.
+#' It is automatically detected by \pkg{ICU}
+#' to match -- by default -- the encoding part of the \code{LC_CTYPE} category
+#' as given by \code{\link{Sys.getlocale}}.
+#'
+#'
+#'
+#' @param str character vector
+#' or an object coercible to a character vector
+#'
+#' @return Returns a character vector of the same length as \code{str}.
+#' Unlike in \code{\link{Encoding}}, possible encodings are:
+#' \code{ASCII}, \code{latin1}, \code{bytes}, \code{native},
+#' and \code{UTF-8}. Additionally, missing values are handled properly.
+#'
+#' This is exactly the same information which is used by
+#' all the functions in \pkg{stringi} to re-encode their inputs.
+#'
+#' @family encoding_management
+#' @export
+stri_enc_mark <- function(str) {
+   .Call("stri_enc_mark", str, PACKAGE="stringi")
+}
